

#include <math.h>
#include "shared.h"
#include "phase3.h"
#include "../src/params.h"
#include "../src/def_params.h"


/*
 * 3-body phase integral for 2 massive and 1 massless particle in CMS frame, with phi2==0
 *
 * p1^mu = (E1, p1*sin(theta1)*cos(phi1), p1*sin(theta1)*sin(phi1), p1*cos(theta1))		mass^2 = m1_sq
 * p2^mu = (p2, p2*sin(theta2), 0, p2*cos(theta2)										mass^2 = 0
 * p3^mu = (E3, -3vec(p1)-3vec(p2))														mass^2 = m3_sq
 *
 * arg_sqrs = sqrt(s)
 * cost1 = cos(theta1)
 * cost2 = cos(theta2)
 * cost12 = cos(angle between p1 & p2)
 */
double phase3(double x0,double x1, double x2, double x3, double m1, double m3, double arg_sqrs,
		double & p1, double & p2, double & E1, double & E3, double & cost1, double & cost2, double & cost12){

	if (arg_sqrs < (m1 + m3)) return 0;
	
	double m1_sq = m1*m1;
	double m3_sq = m3*m3;

	double p1scale = sqrt(kaellen_function(arg_sqrs*arg_sqrs, m1_sq, m3_sq))/arg_sqrs;

	p1 = p1scale/2*x0;
	cost1 = 2*x1-1;
	cost2 = 2*x2-1;
	double phi1 = 2*Pi*x3;
	
	cost12 = sqrt(1-cost1*cost1)*sqrt(1-cost2*cost2)*cos(phi1) + cost1*cost2;

	E1 = sqrt(p1*p1+m1_sq);
	p2 = (m1_sq - m3_sq + arg_sqrs*arg_sqrs - 2*E1*arg_sqrs)/2/(p1*cost12 - E1 + arg_sqrs);
	E3 = sqrt(p1*p1 + p2*p2 + 2*p1*p2*cost12 + m3_sq);

        double s = arg_sqrs*arg_sqrs;
        double thetaarg = (s + m1_sq - m3_sq - 2.*arg_sqrs*E1)*(p1*cost12 + arg_sqrs - E1);
        if ( thetaarg < 0 ) return 0;

	//TODO Restore or document
	double pref = p1scale*p1*p1*p2 / E1 / E3 / 4 / pow(2*Pi,3) / (1 + (p2 + p1*cost12)/E3 ); //Original
	//cout << "fac = " << 1/(1 + (p2 + p1*cost12)/E3 ) <<   "    "  << (p2 + p1*cost12)/E3  << endl;
	//double pref = p1scale*p1*p1*arg_sqrs/2 / E1 / E3 / 4 / pow(2*Pi,3) / (1 + (p2 + p1*cost12)/E3 );

	
	return pref;
}


// reproduce parametrization of Altenkamp et al.
double phase3alt(double x0,double x1, double x2, double x3, double m1, double m3, double arg_sqrs,
		double & p1, double & p2, double & E1, double & E3, double & cost1, double & cost2, double & cost12){

// start translation
        double mymz = m1;
        double mymh = m3;

// this part is in my language (inc. gluons: 1,2; outg. gluon: 3; Higg,Z: h,z or also 4,5)
        double eta = 1.-x0;
        double s = arg_sqrs*arg_sqrs;

        if (s*(1.-eta) < (mymz+mymh)*(mymz+mymh)) return 0;

        double myE1 = arg_sqrs/2.;
        double myE2 = myE1;
        double myE3 = myE1*eta;
        double costheta = 1. - 2.*x2;
        double sintheta = sqrt(1. - costheta*costheta);

// in CM45
        double s45 = x0*s;
        double pzCM45 = sqrt(kaellen_function(mymz*mymz,mymh*mymh,s45))/2./sqrt(s45);
        double coskappa = 1-2*x1;

        double s35 = 1./2./s45 * (
                                   (s + s45)*mymh*mymh + (s45 - mymz*mymz)*(s - s45)
                                   - (1. - 2.*x1)*(s - s45)*sqrt(kaellen_function(mymz*mymz,mymh*mymh,s45))
                                   );

        double EZ = (mymz*mymz + s - s35)/2./sqrt(s);
        double pz = sqrt(EZ*EZ - mymz*mymz);
        double coschi = -1./2./s45/sqrt(kaellen_function(mymz*mymz,s35,s))
                * ( (mymz*mymz - mymh*mymh + s45)*(s - s45)
                    + (1. - 2.*x1)*(s + s45)
                    *sqrt(kaellen_function(mymz*mymz,mymh*mymh,s45)) );
        double sinchi = sqrt(1. - coschi*coschi);

        double cosphi = 1. - 2.*x3;
        double sinphi = sqrt(1. - cosphi*cosphi);

        double t3_overeta = -s/2.*(1-costheta);
        double u3_overeta = -s/2.*(1+costheta);
        double t4 = (myE1-EZ)*(myE1-EZ) - pz*pz*sinchi*sinchi*sinphi*sinphi - pz*pz*(sintheta*coschi+costheta*sinchi*cosphi)*(sintheta*coschi+costheta*sinchi*cosphi) - ( pz*(costheta*coschi - sintheta*sinchi*cosphi) - myE1)*( pz*(costheta*coschi - sintheta*sinchi*cosphi) - myE1);
        double u4 = (myE1-EZ)*(myE1-EZ) - pz*pz*sinchi*sinchi*sinphi*sinphi - pz*pz*(sintheta*coschi+costheta*sinchi*cosphi)*(sintheta*coschi+costheta*sinchi*cosphi) - ( pz*(costheta*coschi - sintheta*sinchi*cosphi) + myE1)*( pz*(costheta*coschi - sintheta*sinchi*cosphi) + myE1);

        double sp12 = s/2.;
        double sp13_overeta = t3_overeta/2.;
        double sp23_overeta = u3_overeta/2.;
        double sp14 = (t4-mymz*mymz)/2.;
        double sp24 = (u4-mymz*mymz)/2.;
        double sp34_overeta = (sqrt(kaellen_function(s45,mymz*mymz,mymh*mymh))*coskappa - mymh*mymh + mymz*mymz + s - eta*s)/(4*(1 - eta));
        double costg = (sp13_overeta-sp23_overeta)/sp12;
        double costz = (sp14-sp24)/arg_sqrs/pz;
        double sintz = sqrt(1.-costz*costz);
        double sintg = sqrt(1.-costg*costg);
        double costzg = (-sp34_overeta*2./arg_sqrs+EZ)/pz;

        double cospz = ((-sp34_overeta*2./arg_sqrs+EZ)/pz-costg*costz)/sintg/sintz;

// finish translation:
//        p1 = pz;
//        p2 = myE3;
//        E1 = EZ;
//        E3 = arg_sqrs - myE3 - EZ;
//        cost1 = costz;
//        cost2 = costg;
//        cost12= costzg;

        double pref = 1./64.*pow(Pi,-4)*sqrt(kaellen_function(mymz*mymz,mymh*mymh,s45))*(1.-x0)/x0/sinphi;

        return pref;
}


#include "../src/undef_params.h"
