
#ifndef CONFIG_H_
#define CONFIG_H_

#include <vector>
#include <string>
#include <map>
#include <iostream>
#include <stdlib.h>


using namespace std;

class configuration {
private:
	//Path to input
	string inputfile;

	//All settings read from the input file
	map<string, string> smap;

	//Read the setting from current input file
	void read_in(bool must_exist);

	//Parse string s as a list: e.g. {<elem1>,<elem2>, ... }   for braces_and_sep = "{}," .
	vector<string> parse_list(const string & s, const char * braces_and_sep) const;

	//Converts to desired type of value. Add specializations below as needed.
	template<typename T>
	T convert(const string & s) const;

public:
	configuration(const string & input_file, bool must_exist = true);
	configuration(const char * input_file, bool must_exist = true);

	void change_input(const string & input_file, bool must_exist = true);

	//Check whether a setting exists
	bool exists(const string & setting) const {return smap.count(setting);};

	template<typename T>
	T get(const string & setting) const;
	template<typename T>
	vector<T> get_1d(const string & setting, const char * braces_and_sep = "{},") const;
	template<typename T>
	vector<vector<T> > get_2d(const string & setting, const char * braces_and_sep = "{},") const;

};

class conversion_error {
public:
	string target_type;
	string input;
	conversion_error(const string & target_type, const string & input):target_type(target_type),input(input) {};
};




/*
 * Specializations of the conversion to different types
 */
//string
template<>
inline string configuration::convert(const string& s) const {
	return s;
}

//bool
template<>
inline bool configuration::convert(const string& s) const {
	if(s.size() == 0) return false;
	if(s.at(0) == '0' || s.at(0) == 'n' || s.at(0) == 'f') return false;
	return true;
}

//int
template<>
inline int configuration::convert(const string& s) const {
	return atoi(s.c_str());
}

//double
template<>
inline double configuration::convert(const string& s) const {
	return atof(s.c_str());
}


// Get a value
template<typename T>
inline T configuration::get(const string& setting) const {
	map<string, string>::const_iterator it = smap.find(setting);
	if(it == smap.end()){
		cerr << "Setting \"" << setting << "\" was not specified in file " << inputfile << "." << endl;
		exit(1);
	}
	T ret;
	try {
		ret = convert<T>(it->second);
	} catch (conversion_error & e) {
		cerr << "Failed to convert setting \"" << setting << "\" to type " << e.target_type << ": " << e.input << endl;
		throw;
	}
	return ret;
}


// Get a vector of values
template<typename T>
inline vector<T> configuration::get_1d(const string& setting, const char * braces_and_sep) const {
	map<string, string>::const_iterator it = smap.find(setting);
	if(it == smap.end()){
		cerr << "Setting \"" << setting << "\" was not specified in file " << inputfile << "." << endl;
		exit(1);
	}
	vector<string> v = parse_list(it->second, braces_and_sep);
	vector<T> ret(v.size());
	for(unsigned int i = 0; i < v.size(); ++i){
		try {
			ret[i] = convert<T>(v[i]);
		} catch (conversion_error & e) {
			cerr << "Failed to convert setting \"" << setting << "\" to type " << e.target_type << ": " << e.input << endl;
			throw;
		}
	}
	return ret;
}

// Get a vector of a vector of values
template<typename T>
inline vector<vector<T> > configuration::get_2d(const string& setting, const char * braces_and_sep) const {
	map<string, string>::const_iterator it = smap.find(setting);
	if(it == smap.end()){
		cerr << "Setting \"" << setting << "\" was not specified in file " << inputfile << "." << endl;
		exit(1);
	}
	vector<string> v1 = parse_list(it->second, braces_and_sep);
	vector<vector<T> > ret(v1.size());
	for(unsigned int i = 0; i < v1.size(); ++i){
		vector<string> v2 = parse_list(v1[i], "{},");
		ret[i] = vector<T>(v2.size());
		for(unsigned int j = 0; j < v2.size(); ++j){
			try {
				ret[i][j] = convert<T>(v2[j]);
			} catch (conversion_error & e) {
				cerr << "Failed to convert setting \"" << setting << "\" to type " << e.target_type << ": " << e.input << endl;
				throw;
			}
		}
	}
	return ret;
}

#endif /* CONFIG_H_ */
