#include "config.h"
#include "hadronic.h"
#include "channel.h"
#include "params.h"
#include "integrate.h"
#include <cmath>
#include <iomanip>
#include "pdfs.h"
#include <atomic>

double hadsqrs;

void hadronic_x_section(bool calculate_sum) {
	//Sum of channels
	double sum = 0;
	double sum_error = 0;
	
	//Save initial parameters
	params tmp_par = global_params;
	
	//Set partonic Sqrt[s]
	hadsqrs = global_params.get_param<double>("hadsqrs");
	
	cout.precision(17);

	for(auto & ch : channel::active_channels){
		cout << endl << "Calculating hadronic cross-section for channel \"" << ch.name << "\"." << endl;

		//Reset parameters
		global_params = tmp_par;

		//Read in channel specific parameters if there are any
		configuration cfg("channels/" + ch.name + ".cfg", false);
		global_params.update_params(cfg);

		//Print parameters for the channel
		string changed_params = global_params.print(tmp_par);
		if(changed_params.size()){
			cout << "Parameters with non-default values for channel " << ch.name << ":" << endl;
			cout << changed_params;
		}

		
		double results[1];
		double errors[1];
		double chi_probs[1];
		std::atomic_uint dropcount(0);
		ch.dropcount = &dropcount;
		run_integral(ch, results, errors, chi_probs, true, &hadronic_integral);
		ch.dropcount = nullptr;

		cout.precision(17); //Increase displayed precision
		cout << left;
		cout << setw(20) << "Result [1/(GeV)^2]: " << results[0] << endl;
		cout << setw(20) << "Error  [1/(GeV)^2]: " << errors[0] << endl;
		cout << setw(20) << "Result [fbarn]: " << 1.0e12*global_params.get_param<double>("hbarcsquared")*results[0] << endl;
		cout << setw(20) << "Error  [fbarn]: " << 1.0e12*global_params.get_param<double>("hbarcsquared")*errors[0] << endl;
		cout << setw(20) << "Relative error: " << ((results[0] == 0) ? 1 : abs(errors[0]/results[0])) << endl;
		cout << setw(20) << "Chi^2 Probability: " << chi_probs[0] << endl;
		cout << setw(20) << "#points dropped: " << dropcount << endl;
		
		if(calculate_sum){
			sum += results[0];
			sum_error = sqrt(sum_error*sum_error + errors[0]*errors[0]);
		}
	}

	//Reset parameters
	global_params = tmp_par;
	
	//Print sum
	if(calculate_sum){
		cout.precision(17); //Increase displayed precision
		cout << endl << "*********************************************" << endl;
		cout << "Sum of all channels: " << endl;
		cout << left;
		cout << setw(20) << "Result [1/(GeV)^2]: " << sum << endl;
		cout << setw(20) << "Error  [1/(GeV)^2]: " << sum_error << endl;
		cout << setw(20) << "Result [fbarn]: " << 1.0e12*global_params.get_param<double>("hbarcsquared")*sum << endl;
		cout << setw(20) << "Error  [fbarn]: " << 1.0e12*global_params.get_param<double>("hbarcsquared")*sum_error << endl;
		cout << setw(20) << "Relative error: " << ((sum == 0) ? 1 : abs(sum_error/sum)) << endl;
	}
}

int hadronic_integral(const int* ndim, const double xs[], const int* ncomp,
		double results[], void* userdata) {
	//ncomp should always be 1 here, only looking at scalar integrands

	//Reinterpret the userdate as the channel info
	const channel * ch = (channel *)userdata;
	
	//Parton momentum fractions, use the last 2 integration variables for these
	double x1 = xs[*ndim-2], x2 = xs[*ndim-1];

	// rescale
	double a = global_params.get_param<double>("shapeparam");;
	double xx1 = pow(x1,a), xx2 = pow(x2,a);
	
	//Avoid numerical divergence
	if(xx1 == 0 || xx2 == 0){
		results[0] = 0;
		return 0;
	}
	
	auto get_pdfs = [xx1,xx2,ch](double fac_scale2){return pdfs2(*ch, xx1, xx2, fac_scale2);};
	double sqrs = hadsqrs * sqrt(xx1*xx2);
	results[0] = ch->cfun(xs, sqrs, get_pdfs, ch);
	
	//Skip pdfs if partonic cross-section is already zero (by kinematic constraints)
	if(results[0] == 0) return 0;
	
	results[0] = results[0] * a * a /x1 /x2;
	
	
	if(std::isnan(results[0])){
		cerr << "NaN encountered as a result for a phase space point. Exiting." << endl;
		for(int i = 0; i < *ndim; ++i){
			cerr << "x[" << i << "] = " << xs[i] << endl;
		}
		//exit(1); //TODO TL 16/09/16: disabled this temporarily for LO_exact channel (LoopTools errors)
		results[0] = 0; //TODO remove this once LT is fixed
	}
	return 0;
}
