
#include "integrate.h"
#include "params.h"
#include "cuba.h"
#include <chrono>


void run_integral(const channel & ch, double results[], double errors[], double chi_probs[], bool hadronic, integrand_t integrand){
	std::string integrator = global_params.get_param<string>("integrator");
	if(integrator == "vegas"){
		run_vegas(ch, results, errors, chi_probs, hadronic, integrand);
	}
	else if(integrator == "cuhre"){
		run_cuhre(ch, results, errors, chi_probs, hadronic, integrand);
	}
	else{
		std::cout << "Invalid integrator. Please specify one of:" << std::endl;
		std::cout << "vegas" << std::endl;
		std::cout << "cuhre" << std::endl;
	}
}




void run_vegas(const channel& ch, double results[], double errors[], double chi_probs[], bool hadronic, integrand_t integrand) {

	channel tmp_ch = ch; //Temporary copy so we can pass it on to Cuba as a non-const pointer

	unsigned int ndim = ch.ndim + (hadronic ? 2 : 0);
	unsigned int ncomp = 1;
	double epsrel = global_params.get_param<double>("epsrel");
	double epsabs = global_params.get_param<double>("epsabs");
	int min_eval = global_params.get_param<int>("min_eval");
	int max_eval = global_params.get_param<int>("max_eval");
	int n_start = global_params.get_param<int>("n_start");
	int n_increase = global_params.get_param<int>("n_increase");
	int n_batch = global_params.get_param<int>("n_batch");
	int n_eval = 0;
	int fail = 0;
	
	cout << "Integrating (Vegas) ..." << endl;
	
	//Timing
	auto time1 = chrono::steady_clock::now();

	Vegas(
			ndim,    //Number of integration variables
			ncomp,      //Number of components of the integrand
			integrand,  //Integrand function
			&tmp_ch,    //Userdata, use it to pass on the channel information to the integrand function
			1,          //Number of points passed to the integrand function in one call
			epsrel,     //Requested relative error
			epsabs,     //Requested absolute error
			0,          //Vegas flags (e.g. verbosity)
			0,          //Random seed
			min_eval,   //Minimal number of integrand evaluations required
			max_eval,   //Maximum number of integrand evaluations allowed
			n_start,    //Number of integrand evaluations per iteration to start with
			n_increase, //Increase in the number of integrand evaluations per iteration
			n_batch,    //Batch size for sampling
			0,          //Grid number
			"",         //Statefile
			NULL,       //Spinning cores pointer
			&n_eval,    //Actual number of integrand evaluations needed
			&fail,      //Error flag
			results,    //Result
			errors,     //Presumed absolute error
			chi_probs   //chi^2 probability that error is not a reliable estimate of the actual error
			);

	//Check for errors

	if(fail < 0){
		cerr << "Integration error: Dimension out of range." << endl;
		cerr << "Cuba error code: " << fail << endl;
		exit(1);
	}

	if(fail > 0){
		cout << "Warning: Accuracy goal not met within specified maximum number of evaluations." << endl;
	}

	cout << "Number of integrand evaluations: " << n_eval << endl;
	
	auto time2 = chrono::steady_clock::now();
	cout.precision(4);
	cout << "Integration time: " << (int)chrono::duration<double>(time2-time1).count() << "s. Per iteration: "
		<< chrono::duration<double, std::milli>(time2-time1).count()/n_eval << "ms" << endl;
	

}


void run_cuhre(const channel& ch, double results[], double errors[], double chi_probs[], bool hadronic, integrand_t integrand) {

	channel tmp_ch = ch; //Temporary copy so we can pass it on to Cuba as a non-const pointer

	unsigned int ndim = ch.ndim + (hadronic ? 2 : 0);
	unsigned int ncomp = 1;
	double epsrel = global_params.get_param<double>("epsrel");
	double epsabs = global_params.get_param<double>("epsabs");
	int min_eval = global_params.get_param<int>("min_eval");
	int max_eval = global_params.get_param<int>("max_eval");
	int n_regions = 0;
	int n_eval = 0;
	int fail = 0;
	
	cout << "Integrating (Cuhre) ..." << endl;
	
	if(ndim < 2){
		cout << "Warning: Cuhre requires integration dimension >=2. An artificial dimension was added for compatibility." << endl;
		cout << "This might increase the required number of points for a given target precision." << endl;
		cout << "To avoid this please choose a different algorithm for the integration." << endl;
		ndim = 2;
	}

	//Timing
	auto time1 = chrono::steady_clock::now();

	Cuhre(
			ndim,    //Number of integration variables
			ncomp,      //Number of components of the integrand
			integrand,  //Integrand function
			&tmp_ch,    //Userdata, use it to pass on the channel information to the integrand function
			1,          //Number of points passed to the integrand function in one call
			epsrel,     //Requested relative error
			epsabs,     //Requested absolute error
			0,          //Cuhre flags (e.g. verbosity)
			min_eval,   //Minimal number of integrand evaluations required
			max_eval,   //Maximum number of integrand evaluations allowed
			0,          //Key = {7,9,11,13}, other->default
			"",         //Statefile
			NULL,       //Spinning cores pointer
			&n_regions, //Actual number of subregions needed
			&n_eval,    //Actual number of integrand evaluations needed
			&fail,      //Error flag
			results,    //Result
			errors,     //Presumed absolute error
			chi_probs   //chi^2 probability that error is not a reliable estimate of the actual error
			);

	//Check for errors

	if(fail < 0){
		cerr << "Integration error: Dimension out of range." << endl;
		cerr << "Cuba error code: " << fail << endl;
		exit(1);
	}

	if(fail > 0){
		cout << "Warning: Accuracy goal not met within specified maximum number of evaluations." << endl;
	}

	cout << "Number of integrand evaluations: " << n_eval << endl;
	
	auto time2 = chrono::steady_clock::now();
	cout.precision(4);
	cout << "Integration time: " << (int)chrono::duration<double>(time2-time1).count() << "s. Per iteration: "
		<< chrono::duration<double, std::milli>(time2-time1).count()/n_eval << "ms" << endl;
}
