
#ifndef PARAMS_H_
#define PARAMS_H_


#include <vector>
#include "config.h"
#include <string.h>
#include <sstream>
#include <list>
#include <sstream>


using std::string;
using std::list;


class params {
public:
	//Values, separated by type
	vector<double> params_d;
	vector<int> params_i;
	vector<string> params_s;


	template<typename T>
	void add_parameter(string name, unsigned int index, T default_value);

	void update_params(const configuration & cfg);

	string print();
	string print(const params & other); //Only prints parameters that are changed compared to the other set of parameters

	template<typename T>
	T get_param(const string & name) const;
	
	void set_param_by_string(const string & s);

private:
	enum type{t_int, t_double, t_string};
	map<string, pair<type, unsigned int> > params_by_name;
	list<string> params_in_order;


	template<typename T>
	vector<T> & get_vec();
	template<typename T>
	type get_type();

};



//Global variable with parameters
extern params global_params;


template<>
inline vector<int> & params::get_vec() {return params_i;};
template<>
inline vector<double> & params::get_vec() {return params_d;};
template<>
inline vector<string> & params::get_vec() {return params_s;};
template<>
inline params::type params::get_type<int>() {return t_int;};
template<>
inline params::type params::get_type<double>() {return t_double;};
template<>
inline params::type params::get_type<string>() {return t_string;};
template<>
inline int params::get_param(const string & name) const{
	map<string, pair<type, unsigned int> >::const_iterator it = params_by_name.find(name);
	if(it == params_by_name.end()){
		cerr << "Parameter \"" << name << "\" (int) was requested but not found. Please define it." << endl;
		exit(1);
	}
	if(it->second.first != t_int){
		cerr << "Parameter \"" << name << "\" was requested as int but has the wrong type." << endl;
		exit(1);
	}
	return params_i[it->second.second];
}
template<>
inline double params::get_param(const string & name) const{
	map<string, pair<type, unsigned int> >::const_iterator it = params_by_name.find(name);
	if(it == params_by_name.end()){
		cerr << "Parameter \"" << name << "\" (double) was requested but not found. Please define it." << endl;
		exit(1);
	}
	if(it->second.first != t_double){
		cerr << "Parameter \"" << name << "\" was requested as double but has the wrong type." << endl;
		exit(1);
	}
	return params_d[it->second.second];
}
template<>
inline string params::get_param(const string & name) const{
	map<string, pair<type, unsigned int> >::const_iterator it = params_by_name.find(name);
	if(it == params_by_name.end()){
		cerr << "Parameter \"" << name << "\" (string) was requested but not found. Please define it." << endl;
		exit(1);
	}
	if(it->second.first != t_string){
		cerr << "Parameter \"" << name << "\" was requested as string but has the wrong type." << endl;
		exit(1);
	}
	return params_s[it->second.second];
}




template<typename T>
inline void params::add_parameter(string name, unsigned int index,
		T default_value) {
	//Ensure the parameter does not exist yet
	if(params_by_name.count(name)){
		cerr << "Parameter \"" << name << "\" was defined more than once." << endl;
		exit(1);
	}

	params_by_name[name] = pair<type, unsigned int>(get_type<T>(), index);
	params_in_order.push_back(name);

	//Ensure that param vector is large enough and set default value
	vector<T> & vec = get_vec<T>();
	if(index >= vec.size()) vec.resize(index+1);
	vec[index] = default_value;
}




inline void params::update_params(const configuration& cfg) {
	for(map<string, pair<type, unsigned int> >::const_iterator it = params_by_name.begin(); it != params_by_name.end(); ++it){
		if(!cfg.exists(it->first)) continue;

		type t = it->second.first;
		switch (t) {
			case t_int:
				params_i[it->second.second] = cfg.get<int>(it->first);
				break;
			case t_double:
				params_d[it->second.second] = cfg.get<double>(it->first);
				break;
			case t_string:
				params_s[it->second.second] = cfg.get<string>(it->first);
				break;
			default:
				cerr << "Unknown parameter type." << endl;
				exit(1);
				break;
		}
	}
}

inline string params::print() {
	stringstream ss;

	for(list<string>::const_iterator it = params_in_order.begin(); it != params_in_order.end(); ++it){
		map<string, pair<type, unsigned int> >::const_iterator p = params_by_name.find(*it);
		type t = p->second.first;
		switch (t) {
			case t_int:
				ss << *it << " = " << params_i[p->second.second] << endl;
				break;
			case t_double:
				ss << *it << " = " << params_d[p->second.second] << endl;
				break;
			case t_string:
				ss << *it << " = " << params_s[p->second.second] << endl;
				break;
			default:
				cerr << "Unknown parameter type." << endl;
				exit(1);
				break;
		}
	}
	return ss.str();
}

inline string params::print(const params & other) {
	stringstream ss;

	for(list<string>::const_iterator it = params_in_order.begin(); it != params_in_order.end(); ++it){
		map<string, pair<type, unsigned int> >::const_iterator p = params_by_name.find(*it);
		type t = p->second.first;
		switch (t) {
			case t_int:
				if(params_i[p->second.second] != other.params_i[p->second.second])
					ss << *it << " = " << params_i[p->second.second] << endl;
				break;
			case t_double:
				if(params_d[p->second.second] != other.params_d[p->second.second])
					ss << *it << " = " << params_d[p->second.second] << endl;
				break;
			case t_string:
				if(params_s[p->second.second] != other.params_s[p->second.second])
					ss << *it << " = " << params_s[p->second.second] << endl;
				break;
			default:
				cerr << "Unknown parameter type." << endl;
				exit(1);
				break;
		}
	}
	return ss.str();
}

//Parses a string of the form "name=value" and assigns the value to the parameter with that name
inline void params::set_param_by_string(const string & s){
	size_t equal = s.find('=');
	if(equal == string::npos){
		cerr << "Invalid parameter assignment: '" << s << "'" << endl;
		cerr << "Correct syntax is    name=value" << endl;
		exit(0);
	}
	
	string name = s.substr(0,equal);
	if(!params_by_name.count(name)){
		cerr << "Invalid parameter assignment: '" << s << "'" << endl;
		cerr << name << " is not a known parameter." << endl;
		exit(0);
	}
	type t = params_by_name[name].first;
	unsigned int index = params_by_name[name].second;
	
	
	string val = s.substr(equal+1);
	
	try{
		switch (t) {
			case t_int:{
				params_i[index] = stoi(val);
				break;
			}
			case t_double:{
				params_d[index] = stod(val);
				break;
			}
			case t_string:{
				params_s[index] = val;
				break;
			}
			default:
				cerr << "Unknown parameter type." << endl;
				exit(1);
				break;
		}
	}
	catch(std::invalid_argument){
		cerr << "Invalid parameter assignment: '" << s << "'" << endl;
		cerr << "The value does not match the parameter type." << endl;
		exit(0);
	}
}

#endif /* PARAMS_H_ */
