#include <iostream>
#include "config.h"
#include "channel.h"
#include <set>
#include "params.h"
#include "partonic.h"
#include "hadronic.h"
#include "pdfs.h"
#include "LHAPDF/LHAPDF.h"
#include "clooptools.h"

using namespace std;


void handle_arguments(int argc, char** argv, string & global_cfg, string & param_cfg, vector<string> & param_args){
	//Set default values
	global_cfg = "xsection.cfg";
	param_cfg = "params.cfg";
	for(int i = 1; i < argc; ++i){
		string s = argv[i];
		if(s.at(0) == '-'){
			if(s.size() < 2){
				cerr << s << " is not a valid argument." << endl;
				exit(1);
			}
			if(i == argc){
				cerr << "option " << s << " must be followed by another argument." << endl;
				exit(1);
			}
			++i;
			string arg = argv[i];
			switch(s.at(1)){
				case 'c':
					global_cfg = arg;
					break;
				case 'p':
					param_cfg = arg;
					break;
				default:
					cerr << "Option not recognised: " << s << endl;
					cerr << "Valid options are -c for config files and -p for parameter files" << endl;
					exit(1);			
			}
		}
		else{
			param_args.push_back(s);
		}
	}
}


int main(int argc,  char** argv) {
	
	//Deal with command line arguments
	vector<string> command_line_params;
	string cfg_file, params_file;
	handle_arguments(argc, argv, cfg_file, params_file, command_line_params);

	//Read configuration
	configuration config(cfg_file);


	//Read in parameter definitions and default values into "global_params"
#include "param_add.h"

	//Read in (potentially) updated values for the parameters
	configuration pcfg(params_file);
	global_params.update_params(pcfg);
	
	//Overwrite parameter values if they are set as command line arguments
	for(auto & s : command_line_params) global_params.set_param_by_string(s);
	

	// init looptools
	ltini();
	setwarndigits(500000);
	seterrdigits(500000);

	//Read in channels
	channel::read_in_channels(config);
	
	//Set pdf sets per channel (the same for all channels at the moment)
	const string pdf_name = config.get<string>("pdf set");
	LHAPDF::PDF * pdf = get_pdf(pdf_name);
	for(auto & ch : channel::active_channels) ch.pdf = pdf;
	
	//Shut LHAPDF up. Otherwise it displays a goodbye message multiple times whenever Cuba is called. 
	//(From a constructor of a global config variable when Cuba (presumably?) uses forks.)
	LHAPDF::setVerbosity(0);

	cout << "alpha_S(m_Z) = " << pdf->alphasQ(91.1876) << endl;
		

	//Print parameters
	cout << "Parameters:" << endl;
	cout << global_params.print();
	
	bool sum_channels = config.get<bool>("sum channels");

	if(config.get<bool>("hadronic")){
		hadronic_x_section(sum_channels);
	}
	else{
		partonic_x_section(sum_channels);
	}
	


	return 0;
}
